"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
// Import sheet and code components
const MitoSheet_1 = __importDefault(require("./MitoSheet"));
const SheetTab_1 = __importDefault(require("./SheetTab"));
const FormulaBar_1 = __importDefault(require("./FormulaBar"));
const MitoToolbar_1 = __importDefault(require("./MitoToolbar"));
const DocumentationSidebar_1 = __importDefault(require("./DocumentationSidebar"));
// Import modals
const ErrorModal_1 = __importDefault(require("./ErrorModal"));
const DownloadModal_1 = __importDefault(require("./DownloadModal"));
const RepeatAnalysisModal_1 = __importDefault(require("./RepeatAnalysisModal"));
// Import css
require("../../css/mito.css");
const INDEX_COLUMN = "index";
var ModalEnum;
(function (ModalEnum) {
    ModalEnum["None"] = "None";
    ModalEnum["Error"] = "Error";
    ModalEnum["RepeatAnalysis"] = "RepeatAnalysis";
    ModalEnum["Download"] = "Download";
})(ModalEnum = exports.ModalEnum || (exports.ModalEnum = {}));
class Mito extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.state = {
            columnSpreadsheetCodeJSON: this.props.columnSpreadsheetCodeJSON,
            dfName: this.props.dfName,
            sheetJSON: this.props.sheetJSON,
            formulaBarValue: this.props.sheetJSON.data[0][0],
            selectedColumn: this.props.sheetJSON.columns[0].toString(),
            selectedRowIndex: 0,
            /*
                note that editingCellColumn and editingCellRowIndex should both either
                be set to these default values or they should be set to valid cell values
            */
            editingCellColumn: "",
            editingCellRowIndex: -1,
            documentationOpen: false,
            modal: ModalEnum.None,
            errorJSON: {
                event: '',
                type: '',
                header: '',
                to_fix: ''
            }
        };
        this.cellFocused = this.cellFocused.bind(this);
        this.handleFormulaBarEdit = this.handleFormulaBarEdit.bind(this);
        this.handleFormulaBarSubmit = this.handleFormulaBarSubmit.bind(this);
        this.sendCellValueUpdate = this.sendCellValueUpdate.bind(this);
        this.setEditingMode = this.setEditingMode.bind(this);
        this.setDocumentation = this.setDocumentation.bind(this);
        this.setModal = this.setModal.bind(this);
        this.getCurrentModalComponent = this.getCurrentModalComponent.bind(this);
    }
    /*
        This function is responsible for turning cell editing mode on and off
        by setting the state of:
            1. editingMode
            2. editingCellColumn
            3. editingCellRowIndex
        This function is called by MitoCellEditor's constructor.

    */
    setEditingMode(on, column, rowIndex) {
        if (on && this.state.editingCellRowIndex === -1) {
            /*
                if this function is turning on cell editing mode (on === true), while
                we're not already in cell editing mode (editingCellRowIndex === -1), then save the cell editor's
                cell. This gets called when the user double clicks on a cell on presses enter
                while a cell is selected and we're not in cell editing mode.
            */
            this.setState({
                editingCellColumn: column,
                editingCellRowIndex: rowIndex
            });
        }
        else if (on && this.state.editingCellRowIndex !== -1) {
            /*
                trying to turn cell editing mode on while cell editing mode is already on
                occurs after the CellFocused function call startEditingCell, causing the cell editor
                to be re-constructed and thus call this function again.
            */
            return;
        }
        else {
            /*
                turn off cell editing mode -- this can occur for a number of reasons.
                See: https://www.ag-grid.com/javascript-grid-cell-editing/#stop-end-editing
                However, we don't allow the user to exit cell editing mode via a new Cell Focus Event
            */
            this.setState({
                editingCellColumn: "",
                editingCellRowIndex: -1
            });
        }
    }
    /*
        We handle this event differently depending on whether we're in or out of cell editing mode
    */
    cellFocused(event) {
        var _a, _b;
        const column = event.column.getColId();
        if (this.state.editingCellRowIndex !== -1) {
            /*
                If we're in editing mode,turn editing back on for the currently edited cell
                This is technically a bit of a hacky solution to pass in the charPress as the selected column
            */
            const selectedColumn = JSON.stringify({ selectedColumn: column });
            /*
                turn on the correct cell editor. see extensive comment in MitoCellEditor.tsx
                constructor about hacking CharPress
            */
            const editingModeParams = {
                rowIndex: this.state.editingCellRowIndex,
                colKey: this.state.editingCellColumn,
                charPress: selectedColumn
            };
            // turn the editing cell's cell editor back on!
            (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.startEditingCell(editingModeParams);
        }
        else {
            // if we're not in editing mode, get the formula for the formula bar
            // if the column is the index column, then we reset the selected cell state
            if (column === INDEX_COLUMN) {
                this.setState({
                    selectedColumn: '',
                    selectedRowIndex: 0,
                    formulaBarValue: ''
                });
                return;
            }
            // otherwise, get the cell's formula to display
            const columnIndex = this.state.sheetJSON.columns.indexOf(column);
            const rowIndex = event.rowIndex;
            const columnFormula = this.state.columnSpreadsheetCodeJSON[column];
            let formulaBarValue = '';
            if (columnFormula !== '') {
                // if the cell has a formula, display it in the formula bar
                formulaBarValue = columnFormula;
            }
            else {
                // otherwise display the value in the formula bar
                formulaBarValue = this.state.sheetJSON.data[rowIndex][columnIndex];
            }
            this.setState({
                selectedColumn: column,
                selectedRowIndex: rowIndex,
                formulaBarValue: formulaBarValue
            });
        }
    }
    handleFormulaBarEdit(e) {
        this.setState({
            formulaBarValue: e.currentTarget.value
        });
    }
    // TODO: do we want a different type of edit for a value change and a formula change or will 
    // we just detect that in the backend and apply the correct edit rules?
    handleFormulaBarSubmit(e) {
        e.preventDefault();
        this.sendCellValueUpdate(this.state.selectedColumn, this.state.formulaBarValue);
    }
    // TODO: this event should be broken out into a formula edit and a value edit
    sendCellValueUpdate(column, newValue) {
        /*
            We don't send the formula to the evaluator while in cell editing mode
            because this function gets called after the CellValueChangedEvent fires
            each time the cell editor is closed.
            
            However, the cell editor closes each time the user uses their mouse
            to reference another column - which isn't a finished update yet!
        */
        if (this.state.editingCellRowIndex === -1) {
            this.props.send({
                'event': 'edit_event',
                'type': 'cell_edit',
                'id': '123',
                'timestamp': '456',
                'address': column,
                'new_formula': newValue
            });
        }
    }
    setDocumentation(documentationOpen) {
        this.setState({ documentationOpen: documentationOpen });
    }
    getCurrentModalComponent() {
        // Returns the JSX.element that is currently, open, and is used
        // in render to display this modal
        switch (this.state.modal) {
            case ModalEnum.None: return react_1.default.createElement("div", null);
            case ModalEnum.Error: return (react_1.default.createElement(ErrorModal_1.default, { errorJSON: this.state.errorJSON, setModal: this.setModal }));
            case ModalEnum.RepeatAnalysis: return (react_1.default.createElement(RepeatAnalysisModal_1.default, { setModal: this.setModal }));
            case ModalEnum.Download: return (react_1.default.createElement(DownloadModal_1.default, { setModal: this.setModal }));
        }
    }
    setModal(modal) {
        this.setState({
            'modal': modal
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    render() {
        // set the selected cell for the cell indicator in the mito toolbar
        const selectedCell = this.state.selectedColumn + (this.state.selectedRowIndex + 1).toString();
        return (react_1.default.createElement("div", { className: "mito-container" },
            react_1.default.createElement("div", { className: "mitosheet" },
                react_1.default.createElement(MitoToolbar_1.default, { sheetJSON: this.state.sheetJSON, selectedCell: selectedCell, send: this.props.send, setDocumentation: this.setDocumentation, setModal: this.setModal, model_id: this.props.model_id }),
                react_1.default.createElement(FormulaBar_1.default, { formulaBarValue: this.state.formulaBarValue, handleFormulaBarEdit: this.handleFormulaBarEdit, handleFormulaBarSubmit: this.handleFormulaBarSubmit }),
                react_1.default.createElement(MitoSheet_1.default, { sheetJSON: this.state.sheetJSON, setEditingMode: this.setEditingMode, cellFocused: this.cellFocused, send: this.props.send, model_id: this.props.model_id, sendCellValueUpdate: this.sendCellValueUpdate }),
                react_1.default.createElement("div", { key: this.state.dfName, className: "sheet-tab-bar" },
                    react_1.default.createElement(SheetTab_1.default, { sheetName: this.state.dfName }))),
            this.getCurrentModalComponent(),
            this.state.documentationOpen &&
                react_1.default.createElement("div", { className: "sidebar" },
                    react_1.default.createElement(DocumentationSidebar_1.default, { setDocumentation: this.setDocumentation }))));
    }
}
exports.default = Mito;
//# sourceMappingURL=Mito.js.map