"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DocumentationSidebar_1 = require("./DocumentationSidebar");
const DocumentationBox_1 = __importDefault(require("./editor/DocumentationBox"));
const SuggestionBox_1 = __importDefault(require("./editor/SuggestionBox"));
const keyboardKeys = {
    ENTER_KEY: "Enter",
    TAB_KEY: "Tab",
    ESCAPE_KEY: "Escape",
    UP: 'ArrowUp',
    DOWN: 'ArrowDown',
    LEFT: 'ArrowLeft',
    RIGHT: 'ArrowRight',
    // These events have IE/Edge differences
    IE_UP: 'Up',
    IE_DOWN: 'Down',
    IE_LEFT: 'Left',
    IE_RIGHT: 'Right'
};
exports.TAKE_SUGGESTION_KEYS = [keyboardKeys.ENTER_KEY, keyboardKeys.TAB_KEY];
const CLOSE_INPUT_KEYS = [keyboardKeys.ENTER_KEY, keyboardKeys.TAB_KEY, keyboardKeys.ESCAPE_KEY];
exports.ARROW_UP_KEYS = [keyboardKeys.UP, keyboardKeys.IE_UP];
exports.ARROW_DOWN_KEYS = [keyboardKeys.DOWN, keyboardKeys.IE_DOWN];
class MitoCellEditor extends react_1.Component {
    constructor(props) {
        super(props);
        // Set the cell's value
        if (props.charPress != null) {
            /*
            We hack the charPress param. The charPress param is usually used to tell the cell editor
            which character was pressed to enter cell editing mode. However, we use also use it to pass
            the column that the user selected with their mouse. If charPress is not null then either:
              1. the user was in cell editing mode and clicked on a column with their mouse. Then charPress
                 is a stringified JSON object of the form:
                 {
                    "selectedColumn": column
                 }
                 where column is the columnID of the column the user selected with their mouse. This columnID
                 should be appended to the cell editor value to be included in the formula.
      
              2. the user entered cell editing mode by pressing a key instead of Enter or double clicking. In this case,
                 the current cell value should be overwritten by charPress
            */
            if (props.charPress.length > 1) {
                // if a column was passed, append it to the cell's value
                const selectedColumn = JSON.parse(props.charPress).selectedColumn;
                this.state = {
                    value: props.value + selectedColumn,
                    input: null,
                    suggestionIndex: 0
                };
            }
            else {
                // if a character was passed, overwrite the cell's value
                // TODO: update the spec with this behavior
                this.state = {
                    value: props.charPress,
                    input: null,
                    suggestionIndex: 0
                };
            }
        }
        else {
            // otherwise keep the original value
            this.state = {
                value: props.value,
                input: null,
                suggestionIndex: 0
            };
        }
        // turn on cell editing mode
        const column = props.colDef.field ? props.colDef.field : "";
        props.setEditingMode(true, column, props.rowIndex);
        this.getValue = this.getValue.bind(this);
        this.isPopup = this.isPopup.bind(this);
        this.handleOnChange = this.handleOnChange.bind(this);
        this.afterGuiAttached = this.afterGuiAttached.bind(this);
        this.getCurrentSuggestions = this.getCurrentSuggestions.bind(this);
        this.getDocumentationBoxFunction = this.getDocumentationBoxFunction.bind(this);
        this.onKeyDown = this.onKeyDown.bind(this);
        this.onMouseEnterSuggestion = this.onMouseEnterSuggestion.bind(this);
        this.selectSuggestion = this.selectSuggestion.bind(this);
        this.getAdjustedIndex = this.getAdjustedIndex.bind(this);
    }
    getValue() {
        return this.state.value;
    }
    /* Make the cell editor a popup, so we can display suggestion/documentation box */
    isPopup() {
        return true;
    }
    /* update the cell value while typing */
    handleOnChange(event) {
        this.setState({ value: event.target.value });
    }
    /*
      This function is called by ag-grid after this component
      is rendered; we simply focus on the input feild rendered
      below so the user can begin typing immediately!
  
      Furthermore, we register the event handlers for this component.
    */
    afterGuiAttached() {
        var _a;
        (_a = this.state.input) === null || _a === void 0 ? void 0 : _a.focus();
    }
    /*
      This function handles key presses on the cell editor input.
  
      If the suggestion box is open currently, than this function first checks
      if this key press effects the suggestion box (e.g. scrolling or selecting a suggestion).
  
      Otherwise, we check if this event closes the cell editor.
  
      NOTE: we event.preventDefault() to stop events from having unintended
      consequences if they cause effects we handle ourselves. For example, this
      prevents TAB from selecting the next form field.
  
      NOTE: the events handled by this functions should correspond to the events
      suppressed in editing mode in the ag-grid, to avoid unintended behavior
    */
    onKeyDown(event) {
        const suggestions = this.getCurrentSuggestions();
        if (suggestions !== undefined) {
            if (exports.TAKE_SUGGESTION_KEYS.includes(event.key)) {
                event.preventDefault();
                // We mod below just as one last safety check for an out of bound index, which we
                // also ensure with the getAdjustedIndex function
                const selectedFunctionObj = suggestions.suggestions[this.state.suggestionIndex % suggestions.suggestions.length];
                const restOfFunction = selectedFunctionObj.function.substring(suggestions.match.length);
                return this.selectSuggestion(restOfFunction);
            }
            else if (exports.ARROW_UP_KEYS.includes(event.key)) {
                event.preventDefault();
                return this.setState((prevState) => {
                    return { suggestionIndex: this.getAdjustedIndex(prevState.suggestionIndex - 1, suggestions) };
                });
            }
            else if (exports.ARROW_DOWN_KEYS.includes(event.key)) {
                event.preventDefault();
                return this.setState((prevState) => {
                    return { suggestionIndex: this.getAdjustedIndex(prevState.suggestionIndex + 1, suggestions) };
                });
            }
        }
        // NOTE: we should always check the close events _last_, in case the keypresses
        // were trying to do other things!
        if (CLOSE_INPUT_KEYS.includes(event.key)) {
            event.preventDefault();
            this.props.setEditingMode(false, this.props.column.getColId(), this.props.rowIndex);
        }
    }
    /*
      This function returns the current suggestions, which are any
      functions who are prepended by the current ending string, which
      must be all capital alphabetic characters.
    */
    getCurrentSuggestions() {
        var _a;
        const suggestionBoxRe = /[A-Z]+/g;
        if (typeof this.state.value == 'string') {
            const functionMatches = (_a = this.state.value) === null || _a === void 0 ? void 0 : _a.match(suggestionBoxRe);
            if (!functionMatches) {
                return undefined;
            }
            const lastMatch = functionMatches[functionMatches.length - 1];
            if (this.state.value.endsWith(lastMatch)) {
                const suggestions = DocumentationSidebar_1.functionDocumentationObjects.filter((funcDocObject) => {
                    return funcDocObject.function.startsWith(lastMatch);
                });
                if (suggestions.length > 0) {
                    return {
                        match: lastMatch,
                        suggestions: suggestions
                    };
                }
            }
        }
        return undefined;
    }
    /*
      Keeps the selected index for the suggestion box inbounds; the index
      should be the index of one of the suggestions!
    */
    getAdjustedIndex(index, suggestions) {
        // Fancy expression to keep it as a valid index
        return Math.min(Math.max(0, index), suggestions.suggestions.length - 1);
    }
    /*
      This function returns the current function that should
      be displayed in the documentation box, based on if the
      documentation box open condition is met.
  
      If the documentation box open condition is not met, this
      returns undefined.
    */
    getDocumentationBoxFunction() {
        var _a;
        // Finds all instances of functions that are not followed by a closing paren
        // e.g. all functions that are still being edited.
        const docBoxRe = /[A-Z]+\((?![^)]*\))/g;
        if (typeof this.state.value == 'string') {
            const functionMatches = (_a = this.state.value) === null || _a === void 0 ? void 0 : _a.match(docBoxRe);
            if (!functionMatches) {
                return undefined;
            }
            // We take the _last_ function that has been written, as this is the funciton
            // being edited currently.
            const lastFunction = functionMatches[functionMatches.length - 1];
            // Strip off the last ( from the function name
            const lastFunctionClean = lastFunction.substring(0, lastFunction.length - 1);
            return DocumentationSidebar_1.functionDocumentationObjects.find((funcDocObject) => {
                return funcDocObject.function === lastFunctionClean;
            });
        }
        return undefined;
    }
    // If you mouseover a suggestion, this selects it
    onMouseEnterSuggestion(suggestionIndex) {
        this.setState({ suggestionIndex: suggestionIndex });
    }
    // Called when a suggestion is selected
    selectSuggestion(restOfFunction) {
        // We fill in the rest of the function, and reset the suggestion box
        this.setState((prevState) => {
            return {
                value: prevState.value + restOfFunction + '(',
                suggestionIndex: 0
            };
        }, () => {
            var _a;
            // We also refocus on the input feild
            (_a = this.state.input) === null || _a === void 0 ? void 0 : _a.focus();
        });
    }
    render() {
        const suggestions = this.getCurrentSuggestions();
        const documentationBoxFunction = this.getDocumentationBoxFunction();
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("input", { ref: (input) => { if (!this.state.input) {
                    this.setState({ input: input });
                } }, className: "ag-cell-auto-height ag-cell-inline-editing", name: "value", value: this.state.value, onChange: this.handleOnChange, onKeyDown: this.onKeyDown, tabIndex: 1 }),
            suggestions !== undefined &&
                react_1.default.createElement(SuggestionBox_1.default, { match: suggestions.match, suggestions: suggestions.suggestions, index: this.state.suggestionIndex, onMouseEnterSuggestion: this.onMouseEnterSuggestion, onSelectSuggestion: this.selectSuggestion }),
            suggestions === undefined && documentationBoxFunction &&
                react_1.default.createElement(DocumentationBox_1.default, { funcDocObject: documentationBoxFunction })));
    }
}
exports.default = MitoCellEditor;
//# sourceMappingURL=MitoCellEditor.js.map