"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
// NOTE: This should be the same as the height defined in widget.css
const SHEET_HEIGHT_PIXELS = 250;
exports.DEFAULT_COLUMN_WIDTH = 125;
exports.headerHeightParams = {
    defaultHeaderHeight: 40,
    charactersPerRow: 9,
    pixelsPerRow: 35 // number of pixels per row
};
/*
    A helper function for getting the width of columns, for all columns in all sheets. If given the prevColumnWidths, then
    will take columnWidths from there - and if not, will default to the DEFAULT_COLUMN_WIDTH.
*/
function getColumnWidthsArray(sheetJSONArray, prevColumnWidthsArray) {
    let changed = prevColumnWidthsArray === undefined || sheetJSONArray.length !== prevColumnWidthsArray.length;
    const newColumnWidthsArray = [];
    for (let i = 0; i < sheetJSONArray.length; i++) {
        const columnHeaders = sheetJSONArray[i].columns;
        const prevColumnWidths = prevColumnWidthsArray ? prevColumnWidthsArray[i] : undefined;
        const newColumnsWidthsResult = getColumnWidths(columnHeaders, prevColumnWidths);
        newColumnWidthsArray.push(newColumnsWidthsResult.columnWidths);
        changed = changed || newColumnsWidthsResult.changed;
    }
    return {
        columnWidthsArray: newColumnWidthsArray,
        changed: changed
    };
}
exports.getColumnWidthsArray = getColumnWidthsArray;
/*
    A helper function for getting the width of a list of columns. If given the prevColumnWidths, then
    will take columnWidths from there - and if not, will default to the DEFAULT_COLUMN_WIDTH.

    Includes a helper variable, changed, which tells if you if anything is different between these objects
*/
function getColumnWidths(columnHeaders, prevColumnWidths) {
    var _a;
    const newColumnWidths = {};
    // We mark these widths as changed if there were no previous widths, or if there are a different number of columns
    let changed = prevColumnWidths === undefined || columnHeaders.length !== ((_a = Object.keys(prevColumnWidths)) === null || _a === void 0 ? void 0 : _a.length);
    for (let i = 0; i < columnHeaders.length; i++) {
        const columnHeader = columnHeaders[i];
        if (prevColumnWidths && prevColumnWidths[columnHeader]) {
            newColumnWidths[columnHeader] = prevColumnWidths[columnHeader];
        }
        else {
            newColumnWidths[columnHeader] = exports.DEFAULT_COLUMN_WIDTH;
            // If there is a new column, then things have changed!
            changed = true;
        }
    }
    return {
        columnWidths: newColumnWidths,
        changed: changed
    };
}
exports.getColumnWidths = getColumnWidths;
function calculateAndSetHeaderHeight(model_id) {
    var _a, _b;
    // we now do some work to find the best height to give the header row
    // first find the largest header
    let maxHeaderLength = 0;
    const gridApi = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(model_id);
    // if the gridApi does not exist, stop
    if (gridApi === undefined) {
        return;
    }
    const rowNodeData = (_b = gridApi.getRowNode('0')) === null || _b === void 0 ? void 0 : _b.data;
    if (rowNodeData === undefined) {
        // This is undefined when the sheet is empty, and thus there aren't
        // any column headers to resize. Thus, we don't need to do anything.
        return;
    }
    Object.keys(rowNodeData).forEach(key => {
        /*
            because each row can only hold 9 characters on average,
            if any words are greater than 9 characters, it will be split into two rows.

            To make sure that we accommodate these special situations, we assume the worst case
            scenario, and add an extra entire line.
        */
        let paddingCharacters = 0;
        getHeaderWords(key).forEach(word => {
            if (word.length > exports.headerHeightParams.charactersPerRow) {
                paddingCharacters += exports.headerHeightParams.charactersPerRow;
            }
        });
        maxHeaderLength = key.length + paddingCharacters > maxHeaderLength ? key.length + paddingCharacters : maxHeaderLength;
    });
    /*
        then set the correct header height...
        we fit on average 10 letters per row with a starting width of 40 pixels
        and we give each row 30 px of space
    */
    gridApi.setHeaderHeight(Math.min(Math.max((maxHeaderLength / exports.headerHeightParams.charactersPerRow - 1) * exports.headerHeightParams.pixelsPerRow, exports.headerHeightParams.defaultHeaderHeight), SHEET_HEIGHT_PIXELS / 3));
    gridApi.refreshHeader();
}
exports.calculateAndSetHeaderHeight = calculateAndSetHeaderHeight;
function getHeaderWords(header) {
    /*
        We split the headers at _, and add _ to the latter half (e.g. first_name -> [first, _name]).

        If consecutive words can be combined and be less than the characters per row, then
        we combine them greedly.

    */
    const headerWords = [];
    const headerWordSplit = header.split(/(?=_)/g);
    let i;
    for (i = 0; i < headerWordSplit.length - 1; i++) {
        const firstWord = headerWordSplit[i];
        const secondWord = headerWordSplit[i + 1];
        if ((firstWord.length + secondWord.length) <= exports.headerHeightParams.charactersPerRow) {
            headerWords.push(firstWord + secondWord);
            i++;
        }
        else {
            headerWords.push(firstWord);
        }
    }
    if (i === headerWordSplit.length - 1) {
        headerWords.push(headerWordSplit[headerWordSplit.length - 1]);
    }
    return headerWords;
}
exports.getHeaderWords = getHeaderWords;
//# sourceMappingURL=gridStyling.js.map